@@ -9,20 +9,12 @@ import logging
 import os.path
 import sys
 import io
+import time
 
 EDGAR_PREFIX = "https://www.sec.gov/Archives/"
 SEP = "|"
 IS_PY3 = sys.version_info[0] >= 3
-
-
-def _worker_count():
-    cpu_count = 1
-    try:
-        cpu_count = len(os.sched_getaffinity(0))
-    except AttributeError:
-        cpu_count = multiprocessing.cpu_count()
-    return cpu_count
-
+REQUEST_BUDGET_MS = 200
 
 def _get_current_quarter():
     return "QTR%s" % ((datetime.date.today().month - 1) // 3 + 1)
@@ -67,13 +59,14 @@ def _skip_header(f):
         f.readline()
 
 
-def _url_get(url):
+def _url_get(url, user_agent):
     content = None
     if IS_PY3:
         # python 3
         import urllib.request
-
-        content = urllib.request.urlopen(url).read()
+        hdr = { 'User-Agent' : user_agent }
+        req = urllib.request.Request(url, headers=hdr)
+        content =urllib.request.urlopen(req).read()
     else:
         # python 2
         import urllib2
@@ -82,7 +75,7 @@ def _url_get(url):
     return content
 
 
-def _download(file, dest, skip_file):
+def _download(file, dest, skip_file, user_agent):
     """
     Download an idx archive from EDGAR
     This will read idx files and unzip
@@ -101,7 +94,7 @@ def _download(file, dest, skip_file):
 
     if url.endswith("zip"):
         with tempfile.TemporaryFile(mode="w+b") as tmp:
-            tmp.write(_url_get(url))
+            tmp.write(_url_get(url, user_agent))
             with zipfile.ZipFile(tmp).open("master.idx") as z:
                 with io.open(dest + dest_name, "w+", encoding="utf-8") as idxfile:
                     _skip_header(z)
@@ -117,7 +110,10 @@ def _download(file, dest, skip_file):
         raise logging.error("python-edgar only supports zipped index files")
 
 
-def download_index(dest, since_year, skip_all_present_except_last=False):
+def _get_millis():
+    return round(time.time() * 1000)
+
+def download_index(dest, since_year, user_agent, skip_all_present_except_last=False):
     """
     Convenient method to download all files at once
     """
@@ -126,18 +122,21 @@ def download_index(dest, since_year, skip_all_present_except_last=False):
 
     tasks = _quarterly_idx_list(since_year)
     logging.info("%d index files to retrieve", len(tasks))
-
-    worker_count = _worker_count()
-    logging.debug("worker count: %d", worker_count)
-    pool = multiprocessing.Pool(worker_count)
-
+    last_download_at = _get_millis()
     for i, file in enumerate(tasks):
         skip_file = skip_all_present_except_last
         if i == 0:
             # First one should always be re-downloaded
             skip_file = False
-        pool.apply_async(_download, (file, dest, skip_file))
+        # naive: 200ms or 5QPS serialized
+        start = _get_millis()
+        _download(file, dest, skip_file, user_agent)
+        elapsed = _get_millis() - start
+        if elapsed < REQUEST_BUDGET_MS:
+            sleep_for = REQUEST_BUDGET_MS-elapsed
+            logging.info("sleeping for %dms because we are going too fast (previous request took %dms", sleep_for, elapsed)
+            time.sleep(sleep_for/1000)
+        last_download_at = _get_millis()
+
 
-    pool.close()
-    pool.join()
     logging.info("complete")
